/*
    psftools: Manipulate console fonts in the .PSF format
    Copyright (C) 2000  John Elliott

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* Convert a PSF file to a C include file. Used while building zx2psf. */

#include "cnvshell.h"
#include "psflib.h"

char *cnv_progname = "PSF2INC";

static char helpbuf[2000];
static int v1 = 0, v2 = 0;

char *cnv_set_option(int ddash, char *variable, char *value)
{
    if (!stricmp(variable, "psf1"))   { v1 = 1; return NULL; }
    if (!stricmp(variable, "psf2"))   { v2 = 1; return NULL; }
    if (strlen(variable) > 2000) variable[2000] = 0;
    sprintf(helpbuf, "Unknown option: %s\n", variable);
    return helpbuf;
}

/* Return help string */
char *cnv_help(void)
    {
    sprintf(helpbuf, "Syntax: %s psffile incfile { options }\n\n", cnv_progname);
    strcat (helpbuf, "Options: \n"
                     "    --psf1         Force PSF1 format\n"
                     "    --psf2         Force PSF2 format\n");

    return helpbuf;
    }

static psf_dword filepos;

int write_printf(PSFIO *io, psf_byte b)
{
	static psf_dword charbase, ch, x, cb;
	int ct = 0;
	static psf_byte pch;

	if (!filepos) 
	{
		fprintf(io->data.fp, "   ");
		charbase = io->psf->psf_hdrlen;
		ch = 0;
		x = 0;
		cb = 0;
	}
	fprintf(io->data.fp, "0x%02x, ", b);
	if (io->psf->psf_magic == PSF1_MAGIC) switch(filepos)
	{
		case 1: fprintf(io->data.fp,"  /* Magic */\n   ");     x = 0; cb = 0; ct = 1; break;
		case 2: fprintf(io->data.fp,"  /* Type */\n   ");      x = 0; cb = 0; ct = 1; break;
		case 3: fprintf(io->data.fp,"  /* Char size */\n   "); x = 0; cb = 0; ct = 1; break;
	}
	if (io->psf->psf_magic == PSF_MAGIC) switch(filepos)
	{
		case  3: fprintf(io->data.fp,"  /* Magic */\n   ");        x = 0; cb = 0; ct = 1; break;
		case  7: fprintf(io->data.fp,"  /* Version */\n   ");      x = 0; cb = 0; ct = 1; break;
		case 11: fprintf(io->data.fp,"  /* Header size */\n   ");  x = 0; cb = 0; ct = 1; break;
		case 15: fprintf(io->data.fp,"  /* Flags */\n   ");        x = 0; cb = 0; ct = 1; break;
		case 19: fprintf(io->data.fp,"  /* No. of chars */\n   "); x = 0; cb = 0; ct = 1; break;
		case 23: fprintf(io->data.fp,"  /* Char length */\n   ");  x = 0; cb = 0; ct = 1; break;
		case 27: fprintf(io->data.fp,"  /* Char width */\n   ");   x = 0; cb = 0; ct = 1; break;
		case 31: fprintf(io->data.fp,"  /* Char height */\n   ");  x = 0; cb = 0; ct = 1; break;
	}
	++filepos;
	if (ct) { pch = b; return PSF_E_OK; }
	if (ch >= io->psf->psf_length)	/* Unicode dir */
	{
		if (io->psf->psf_magic == PSF_MAGIC && b == 0xFF)
		{
			fprintf(io->data.fp, "\n   ");
			x = 0;
		}
		if (io->psf->psf_magic == PSF1_MAGIC && b == 0xFF && pch == 0xFF)
		{
			fprintf(io->data.fp, "\n   ");
			x = 0;
		}
	}

	cb++;
	if (cb == io->psf->psf_charlen && ch < io->psf->psf_length)
	{
		fprintf(io->data.fp, "  /* %ld */\n   ", ch++);
		cb = x = 0;
	}
	else x++;
	if (x == 8)
	{
		fprintf(io->data.fp, "\n   ");
		x = 0;
	}	

	pch = b;
	return PSF_E_OK;;
}




char *cnv_execute(FILE *infile, FILE *outfile)
{	
	int rv;
	PSF_FILE psf;
	PSFIO psfio;
	char *s = NULL;

	filepos = 0;
	psf_file_new(&psf);
	rv = psf_file_read(&psf, infile);	

	if (rv != PSF_E_OK) return psf_error_string(rv);

	psfio.psf = &psf;
	psfio.readfunc  = NULL;
	psfio.writefunc = write_printf;
	psfio.data.fp = outfile; 

	if (v1) psf_force_v1(&psf);
	if (v2) psf_force_v2(&psf);

	if ((rv = psf_write(&psfio))) s = psf_error_string(rv);

	psf_file_delete(&psf);
	return s;
}

