/*
    psftools: Manipulate console fonts in the .PSF format
    Copyright (C) 2003  John Elliott

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include "cnvshell.h"
#include "psflib.h"

/* Convert a PSF font to a raw font (as used by DOS font editors etc.) 
 *
 * Not much more than "dd if=psf of=font bs=1 skip=4"; there is an option
 * to write only the first 256 characters of a 512-character font.
 *
 */

static char helpbuf[2048];
static int first  = -1;
static int last   = -1;
static int doflip = 0;

static PSF_FILE psf;

/* Program name */
char *cnv_progname = "PSF2RAW";

/* ddash = 1 if option started with a double-dash; else 0 */
/* Return NULL if OK, else error string */
char *cnv_set_option(int ddash, char *variable, char *value)
    {
    if (!stricmp(variable, "first"))  { first = atoi(value); return NULL; }
    if (!stricmp(variable, "last"))   { last = atoi(value); return NULL; }
    if (!stricmp(variable, "256"))    { first = 0; last = 255; return NULL; }
    if (!stricmp(variable, "flip"))   { doflip = 1; return NULL; }
    if (strlen(variable) > 2000) variable[2000] = 0;
    sprintf(helpbuf, "Unknown option: %s\n", variable);
    return helpbuf;
    }


/* Return help string */
char *cnv_help(void)
    {
    sprintf(helpbuf, "Syntax: %s psffile rawfnt { options }\n\n", cnv_progname);
    strcat (helpbuf, "Options: \n"
		     "    --first=n Start with character n\n"
                     "    --last=n  Finish with character n\n"
                     "    --flip    Mirror all bytes as they are written\n"
                     "    --256     Equivalent to --first=0 --last=255\n");
            
    return helpbuf;
    }


char *cnv_execute(FILE *infile, FILE *outfile)
{	
	int rv, n, max;
	psf_dword ch, f, l;

	psf_file_new(&psf);
        rv = psf_file_read(&psf, infile);

	if (rv != PSF_E_OK) return psf_error_string(rv);

	/* Most systems that use raw fonts depend on the font being 1 byte 
	 * wide */
	if (psf.psf_width > 8) 
		fprintf(stderr, "Warning: Font is wider than 8 bits.\n");

	f = (first >= 0) ? first : 0;
	l = (last  >= 0) ? last  : (psf.psf_length - 1);
	if (l >= psf.psf_length) l = psf.psf_length;

	if (doflip)
	{
		max = (last+1) * psf.psf_charlen;
		for (n = first * psf.psf_charlen; n < max; n++)
		{
			psf.psf_data[n] = flip(psf.psf_data[n]);
		}
	}
	for (ch = f; ch <= l; ch++)
	{
		if (fwrite(psf.psf_data + ch * psf.psf_charlen, 
			1, psf.psf_charlen, outfile) < psf.psf_charlen)
		{
			psf_file_delete(&psf);
			return "Could not write to output.";
		}
	}	
	psf_file_delete(&psf);	
	return NULL;
}

