/*
    psftools: Manipulate console fonts in the .PSF format
    Copyright (C) 2003  John Elliott

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <stdio.h>
#include <string.h>
#ifndef CPM
# ifndef __PACIFIC__
#  include <errno.h>
# endif
#endif
#include "zxflib.h"

/* Functions to manipulate Spectrum +3DOS fonts */

void zxf_file_new(ZXF_FILE *f)
{
	memset(f->zxf_header, 0, sizeof(f->zxf_header));
	memset(f->zxf_data,   0, sizeof(f->zxf_data));

/* Format of +3DOS font header:
0	DB	'PLUS3DOS',26	;+3DOS header magic
9	DB	1,0		;+3DOS header version; always 1.0
11	DD	filesize	;inclusive of header
15	DB	filetype	;fonts are always type 3
16	DW	datasize	;file size less header
18	DW	load address	;Not important for fonts
20	DW	variables	;Not used for fonts
22	DB	spare		;Spare; we will set it to 'f' for "Font".
-- zeroes --
127	DB	checksum	;128-byte checksum of header */

	memcpy(f->zxf_header, ZXF_MAGIC, 11);
	f->zxf_header[11] = 0x80; f->zxf_header[12] = 0x03;  /* Length 0x380 */
	f->zxf_header[16] = 0x00; f->zxf_header[17] = 0x03;  /* Size   0x300 */
	f->zxf_header[18] = 0x00; f->zxf_header[19] = 0x3D;  /* Load addr */ 
	f->zxf_header[22] = 'f';

	f->zxf_header[15] = 0;	/* Mark file as empty */
}

/* Initialise the structure for a new font*/
zxf_errno_t zxf_file_create(ZXF_FILE *f)
{
	zxf_file_delete(f);
	f->zxf_header[15] = 3;
	return ZXF_E_OK;
}

/* Load a Spectrum +3 font from a stream */
zxf_errno_t zxf_file_read  (ZXF_FILE *f, FILE *fp)
{
	int n;
	zxf_byte b = 0;
	zxf_byte head[128];

	if (fread(head, 1, 128, fp) < 128) 
	{
		if (ferror(fp)) return ZXF_E_ERRNO;
		return ZXF_E_NOTP3FNT;
	}
	if (memcmp(head, ZXF_MAGIC, 11)) return ZXF_E_NOTP3FNT;
	for (n = 0; n < 127; n++) b += head[n];
	if (b != head[n]) return ZXF_E_NOTP3FNT;	/* Not +3DOS file */

	if (head[11] != 0x80     || head[12] != 0x03 ||
            head[13] || head[14] || head[15] != 0x03 ||
	    head[16]             || head[17] != 0x03) return ZXF_E_NOTP3FNT;
	/* Length or filetype is wrong */

	memcpy(f->zxf_header, head, 128);
	if (fread(f->zxf_data, 1, 768, fp) < 768)
	{
		if (ferror(fp)) return ZXF_E_ERRNO;
		return ZXF_E_NOTP3FNT;
	}
	return ZXF_E_OK;
}


/* Write Spectrum +3 font to stream */
zxf_errno_t zxf_file_write (ZXF_FILE *f, FILE *fp)
{
	int n;
	zxf_byte b = 0;

	if (!f->zxf_header[15]) return ZXF_E_EMPTY;

	/* Checksum the header */
	for (n = 0; n < 127; n++) b += f->zxf_header[n];
	f->zxf_header[n] = b;

	/* Write out the data */
	if (fwrite(f->zxf_header, 1, 128, fp) < 128) return ZXF_E_ERRNO;
	if (fwrite(f->zxf_data,   1, 768, fp) < 768) return ZXF_E_ERRNO;
	return ZXF_E_OK;
}


/* Free any memory associated with a +3 font file (dummy) */
void zxf_file_delete (ZXF_FILE *f)
{
	zxf_file_new(f);
}

char *zxf_error_string(zxf_errno_t err)
{
        switch(err)
        {
                case ZXF_E_OK:          return "No error";
                case ZXF_E_NOTP3FNT:    return "File is not a Spectrum +3 font file";
                case ZXF_E_EMPTY:       return "Attempt to save an empty file";
                case ZXF_E_ERRNO:       return strerror(errno);
        }
        return "Unknown error";

}



